@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2010 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2010 Laszlo Molnar
+   Copyright (C) 1996-2011 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2011 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -202,6 +202,7 @@ unsigned PackArmPe::processImports() // pass 1
         unsigned   iat;
         LE32       *lookupt;
         unsigned   npos;
+        unsigned   original_position;
         bool       isk32;
 
         static int __acc_cdecl_qsort compare(const void *p1, const void *p2)
@@ -210,6 +211,8 @@ unsigned PackArmPe::processImports() // pass 1
             const udll *u2 = * (const udll * const *) p2;
             if (u1->isk32) return -1;
             if (u2->isk32) return 1;
+            if (!*u1->lookupt) return 1;
+            if (!*u2->lookupt) return -1;
             int rc = strcasecmp(u1->name,u2->name);
             if (rc) return rc;
             if (u1->ordinal) return -1;
@@ -236,11 +239,12 @@ unsigned PackArmPe::processImports() // pass 1
         dlls[ic].iat = im->iat;
         dlls[ic].lookupt = (LE32*) (ibuf + (im->oft ? im->oft : im->iat));
         dlls[ic].npos = 0;
+        dlls[ic].original_position = ic;
         dlls[ic].isk32 = strcasecmp(kernel32dll,dlls[ic].name) == 0;
 
         soimport += strlen(dlls[ic].name) + 1 + 4;
 
-        for (LE32 *tarr = dlls[ic].lookupt; *tarr; tarr++)
+        for (IPTR_I(LE32, tarr, dlls[ic].lookupt); *tarr; tarr += 1)
         {
             if (*tarr & 0x80000000)
             {
@@ -252,10 +256,11 @@ unsigned PackArmPe::processImports() // pass 1
             }
             else
             {
-                unsigned len = strlen(ibuf + *tarr + 2);
+                IPTR_I(const upx_byte, n, ibuf + *tarr + 2);
+                unsigned len = strlen(n);
                 soimport += len + 1;
                 if (dlls[ic].shname == NULL || len < strlen (dlls[ic].shname))
-                    dlls[ic].shname = ibuf + *tarr + 2;
+                    dlls[ic].shname = n;
             }
             soimport++; // separator
         }
@@ -343,8 +348,10 @@ unsigned PackArmPe::processImports() // pass 1
     for (ic = 0; ic < dllnum; ic++)
     {
         LE32 *tarr = idlls[ic]->lookupt;
+#if 0 && ENABLE_THIS_AND_UNCOMPRESSION_WILL_BREAK
         if (!*tarr)  // no imports from this dll
             continue;
+#endif
         set_le32(ppi,idlls[ic]->npos);
         set_le32(ppi+4,idlls[ic]->iat - rvamin);
         ppi += 8;
@@ -412,7 +419,7 @@ unsigned PackArmPe::processImports() // pass 1
         for (ic = 0; ic < dllnum; ic++, im++)
         {
             memset(im,FILLVAL,sizeof(*im));
-            im->dllname = ptr_diff(idlls[ic]->name,ibuf); // I only need this info
+            im->dllname = ptr_diff(dlls[idlls[ic]->original_position].name,ibuf);
         }
     }
     else
